//Shader der GaussFilterung einer Textur macht, von DelphiGL uebernommen. 
//Cool: Kann je nachdem, wie ich uShift setze, entscheiden, ob horizontal oder vertikal 
//gefiltert wird (also: vec2(0, wert) oder (wert,0). 
//Blending kann auch via uniform gesteuert werden. (wenn == 0, wird nur gefiltert)

#version 150
in vec2 UV;

uniform sampler2D renderedTexture;
uniform sampler2D alphaTexture;
uniform int mipmap_level; 
uniform vec2 uShift;
uniform int blend; 

const int gaussRadius = 11;
const float gaussFilter[gaussRadius] = float[gaussRadius](
	0.0402,0.0623,0.0877,0.1120,0.1297,0.3362,0.1297,0.1120,0.0877,0.0623,0.0402
);

out vec4 colorRender;
out vec4 colorAlpha;


void main(){
	colorRender = vec4 (0,0,0,1.0);
	colorRender.rgb = texture( renderedTexture, UV).rgb ;

	vec2 texCoord = UV - float(int(gaussRadius/2)) * uShift;
	vec3 color = vec3(0.0, 0.0, 0.0); 
	for (int i=0; i<gaussRadius; ++i) { 
		color += gaussFilter[i] * texture2D(alphaTexture, texCoord).rgb;
		texCoord += uShift;
	}
	colorAlpha = vec4(color.x, color.y, color.z ,1.0);
	
   
  if(blend == 1){
	//Additives blenden FTW. Simpel und viel effekt.
	 colorRender.x = min(1.0, colorRender.x+colorAlpha.x); 
	 colorRender.y = min(1.0, colorRender.y+colorAlpha.y); 
	 colorRender.z = min(1.0, colorRender.z+colorAlpha.z); 
	 //alternativ: softeres blending.
	 //colorRender = clamp((colorAlpha + colorRender) - (colorAlpha * colorRender), 0.0, 1.0);
  }
	//wenn man nur den alphakanal sehen will, einfach auskommentieren 
   //colorRender=colorAlpha;  
	
	
}