#version 150

// from OpenGL SuperBible & http://www.lighthouse3d.com/tutorials/glsl-tutorial/the-normal-matrix/

vec3 lightColor = vec3(1.0, 1.0, 1.0); // Color of that point light

float ambientFactor = 0.25;
float diffFactor = 0.7;

uniform float specFactor;
uniform float materialSpecFactor;
uniform vec3 defaultMaterialColor;

uniform samplerCube cm_z_tex; //cube map
uniform vec4 l_pos; //world space light position
uniform vec2 near_far; // near and far plane for cm-cams

uniform float opacity;

in vec4 WS_pos; // world space position

smooth in vec3 worldNormal;
smooth in vec3 worldLightDir;
in vec3 viewVector;

out vec4 colorRender;
out vec4 colorAlpha;

void main()
{	
	// calculate vector from surface point to light position
	// (both positions are given in world space)
	vec3 cm_lookup_vec = WS_pos.xyz - l_pos.xyz;

	// WS 'dist-to-lightsource' for current fragment
	float curr_fragment_dist_to_light = length(cm_lookup_vec);

	float eps = 0.05; // add a small offset (adjust as needed)
	float shadowFac = 1;
	
	vec3 l1 = cm_lookup_vec;
	float m1 = max(max(l1.x, l1.y), l1.z);
	l1 = l1 - vec3(m1, m1, m1);
	l1 = l1 / (l1 + vec3(0.001, 0.001, 0.001));
	l1 = l1 * (1.0/100.0);
	
	vec3 offset1 = l1;
	vec3 offset2 = -l1;
	vec3 offset3 = l1;
	if (abs(offset3.x) >= 0.001){
		offset3.x = offset3.x * -1;	
	}
	else{
		offset3.y = offset3.y*-1;
	}
	vec3 offset4 = -offset3;

	vec3 offset5 = (offset4 + offset1) / 2.0;
	vec3 offset6 = (offset1 + offset3) / 2.0;
	vec3 offset7 = (offset3 + offset2) / 2.0;
	vec3 offset8 = (offset2 + offset4) / 2.0;

	float smallest_dist_to_light1 = texture(cm_z_tex, cm_lookup_vec + offset1).r;
	smallest_dist_to_light1 *= near_far.y;
	float smallest_dist_to_light2 = texture(cm_z_tex, cm_lookup_vec + offset2).r;
	smallest_dist_to_light2 *= near_far.y;
	float smallest_dist_to_light3 = texture(cm_z_tex, cm_lookup_vec + offset3).r;
	smallest_dist_to_light3 *= near_far.y;
	float smallest_dist_to_light4 = texture(cm_z_tex, cm_lookup_vec + offset4).r;
	smallest_dist_to_light4 *= near_far.y;

	float smallest_dist_to_light5 = texture(cm_z_tex, cm_lookup_vec + offset5).r;
	smallest_dist_to_light5 *= near_far.y;
	float smallest_dist_to_light6 = texture(cm_z_tex, cm_lookup_vec + offset6).r;
	smallest_dist_to_light6 *= near_far.y;
	float smallest_dist_to_light7 = texture(cm_z_tex, cm_lookup_vec + offset7).r;
	smallest_dist_to_light7 *= near_far.y;
	float smallest_dist_to_light8 = texture(cm_z_tex, cm_lookup_vec + offset8).r;
	smallest_dist_to_light8 *= near_far.y;
		
	if(smallest_dist_to_light1+eps < curr_fragment_dist_to_light){
		shadowFac = shadowFac - 0.125;
	}
	if(smallest_dist_to_light2+eps < curr_fragment_dist_to_light){
		shadowFac = shadowFac - 0.125;
	}
	if(smallest_dist_to_light3+eps < curr_fragment_dist_to_light){
		shadowFac = shadowFac - 0.125;
	}
	if(smallest_dist_to_light4+eps < curr_fragment_dist_to_light){
		shadowFac = shadowFac - 0.125;
	}
	if(smallest_dist_to_light5+eps < curr_fragment_dist_to_light){
		shadowFac = shadowFac - 0.125;
	}
	if(smallest_dist_to_light6+eps < curr_fragment_dist_to_light){
		shadowFac = shadowFac - 0.125;
	}
	if(smallest_dist_to_light7+eps < curr_fragment_dist_to_light){
		shadowFac = shadowFac - 0.125;
	}
	if(smallest_dist_to_light8+eps < curr_fragment_dist_to_light){
		shadowFac = shadowFac - 0.125;
	}	

	// Dot product gives us diffuse intensity
	float diff = max(0.0, dot(normalize(worldNormal), normalize(worldLightDir))) * shadowFac;

	// Multiply intensity by diffuse color, force alpha to 1.0
	colorRender = vec4(diff * diffFactor * lightColor * defaultMaterialColor, 1.0);

	// Add in ambient light
	colorRender += vec4(ambientFactor * lightColor, 1.0);

	// Specular Light
	vec3 vReflection = normalize(reflect(-normalize(worldLightDir), normalize(worldNormal)));
	float spec = max(0.0, dot(normalize(viewVector), vReflection)) * shadowFac;

		// If the diffuse light is zero, don't even bother with the pow function
	if(diff != 0) {
		float fSpec = pow(spec, materialSpecFactor);
		colorRender.rgb += (fSpec * specFactor * lightColor * defaultMaterialColor);
	}

	colorRender.a = opacity; 
	colorAlpha = vec4(0.0,0.0,0.0,0.0);
}